@@ -1,51 +1,55 @@
-import dataclasses
 import json
 import logging
 import pathlib
 import sys
 import typing
 
-import jsonlog
-import jsonschema
+import pydantic
 
-from .colours import Colour, ColourMap
-from .key import Key
-from .pattern import KeyValuePattern, TemplatePattern
+import jsonlog
+import jsonlog_cli.colours
+import jsonlog_cli.pattern
 
 log = logging.getLogger(__name__)
 
 DEFAULT_KEYVALUES = {
-    "default": KeyValuePattern(multiline_keys=(Key("traceback"), Key("stacktrace"))),
-    "elasticsearch": KeyValuePattern(
+    "default": jsonlog_cli.pattern.KeyValuePattern(
+        multiline_keys=("traceback", "stacktrace")
+    ),
+    "elasticsearch": jsonlog_cli.pattern.KeyValuePattern(
         priority_keys=(
-            Key("timestamp"),
-            Key("level"),
-            Key("type"),
-            Key("component"),
-            Key("cluster.name"),
-            Key("node.name"),
-            Key("message"),
+            "timestamp",
+            "level",
+            "type",
+            "component",
+            "cluster.name",
+            "node.name",
+            "message",
         ),
-        multiline_keys=(Key("stacktrace"),),
+        multiline_keys=("stacktrace",),
         multiline_json=True,
     ),
-    "jsonlog": KeyValuePattern(
-        priority_keys=(Key("timestamp"), Key("level"), Key("name"), Key("message")),
-        multiline_keys=(Key("traceback"),),
+    "jsonlog": jsonlog_cli.pattern.KeyValuePattern(
+        priority_keys=("timestamp", "level", "name", "message"),
+        multiline_keys=("traceback",),
     ),
-    "snyk": KeyValuePattern(
-        priority_keys=(Key("time"), Key("msg"), Key("reason.response.body.message")),
-        multiline_keys=(Key("__json__"),),
-        colours=ColourMap({20: Colour(fg="cyan"), 50: Colour(fg="red")}),
+    "snyk": jsonlog_cli.pattern.KeyValuePattern(
+        priority_keys=("time", "msg", "reason.response.body.message"),
+        multiline_keys=("__json__",),
+        colours={
+            20: jsonlog_cli.colours.Colour(fg="cyan"),
+            50: jsonlog_cli.colours.Colour(fg="red"),
+        },
     ),
-    "jaeger": KeyValuePattern(multiline_keys=(Key("errorVerbose"), Key("stacktrace"))),
-    "vault": KeyValuePattern(
-        level_key=Key("@level"),
-        priority_keys=(Key("@timestamp"), Key("@module"), Key("@message")),
+    "jaeger": jsonlog_cli.pattern.KeyValuePattern(
+        multiline_keys=("errorVerbose", "stacktrace")
+    ),
+    "vault": jsonlog_cli.pattern.KeyValuePattern(
+        level_key="@level", priority_keys=("@timestamp", "@module", "@message"),
     ),
 }
 DEFAULT_TEMPLATES = {
-    "default": TemplatePattern(format="{__line__}"),
+    "default": jsonlog_cli.pattern.TemplatePattern(format="{__line__}"),
 }
 CONFIG_SCHEMA = {
     "type": "object",
@@ -71,10 +75,9 @@ CONFIG_SCHEMA = {
 }
 
 
-@dataclasses.dataclass()
-class Config:
-    keyvalues: typing.Dict[str, KeyValuePattern]
-    templates: typing.Dict[str, TemplatePattern]
+class Config(pydantic.BaseModel):
+    keyvalues: typing.Dict[str, jsonlog_cli.pattern.KeyValuePattern]
+    templates: typing.Dict[str, jsonlog_cli.pattern.TemplatePattern]
 
     @classmethod
     def load(cls, filename: str) -> "Config":
@@ -88,23 +91,19 @@ class Config:
         )
 
         if path.exists():
-            config.read(path)
+            loaded = config.read(path)
+            config.templates.update(loaded)
+            config.keyvalues.update(loaded.keyvalues)
 
         return config
 
-    def read(self, path: pathlib.Path) -> None:
+    @staticmethod
+    def read(path: pathlib.Path) -> "Config":
         log.info(
             "Reading configuration from file",
             extra={"path": str(path), "exists": path.exists()},
         )
-        instance = json.loads(path.read_text(encoding="utf-8"))
-        jsonschema.validate(instance=instance, schema=CONFIG_SCHEMA)
-
-        for k, v in instance.get("keyvalues", {}).items():
-            self.keyvalues[k] = KeyValuePattern(**v)
-
-        for k, v in instance.get("templates", {}).items():
-            self.templates[k] = TemplatePattern(**v)
+        return Config(**json.loads(path.read_text(encoding="utf-8")))
 
 
 def configure_logging(path: str) -> None:
