@@ -1,28 +1,34 @@
-import dataclasses
 import itertools
 import json
 import typing
 
-from .colours import Colour, ColourMap
-from .key import Key
-from .record import Record, RecordFormatter, RecordKey, RecordValue
+import pydantic
+
+from .colours import Colour
+from .record import Record, RecordKey, RecordValue
 from .text import wrap_and_style_lines
+from .types import Value
 
 Level = typing.Optional[str]
 P = typing.TypeVar("P", bound="Pattern")
 
 
-@dataclasses.dataclass(frozen=True)
-class Pattern(RecordFormatter):
-    colours: ColourMap = dataclasses.field(default=ColourMap.default(), repr=False)
-    level_key: Key = dataclasses.field(default=Key("level"))
-    multiline_json: bool = dataclasses.field(default=False)
-    multiline_keys: typing.Sequence[Key] = dataclasses.field(default=())
+class Pattern(pydantic.BaseModel):
+    colours: typing.Mapping[Value, Colour] = {
+        "info": Colour(fg="cyan"),
+        "warning": Colour(fg="yellow"),
+        "warn": Colour(fg="yellow"),
+        "error": Colour(fg="red"),
+        "critical": Colour(fg="red", bold=True),
+        "fatal": Colour(fg="red", bold=True),
+    }
+    level_key: str = "level"
+    multiline_json: bool = False
+    multiline_keys: typing.Sequence[str] = ()
 
     def replace(self: P, **changes: typing.Optional[typing.Any]) -> P:
         """Return a copy of the pattern with fields replaced."""
-        changes = {k: v for k, v in changes.items() if v is not None}
-        return dataclasses.replace(self, **changes)
+        return self.copy(update={k: v for k, v in changes.items() if v is not None})
 
     def format_record(self, record: Record) -> str:
         message = self.format_message(record)
@@ -40,7 +46,7 @@ class Pattern(RecordFormatter):
 
     def format_multiline(self, record: Record) -> typing.Iterator[str]:
         for key in self.multiline_keys:
-            value = record.extract(key.name)
+            value = record.extract(key)
             if value:
                 string = self.format_multiline_value(value)
                 yield wrap_and_style_lines(string, dim=True)
@@ -62,15 +68,26 @@ class Pattern(RecordFormatter):
         return json.dumps(value, indent=2)
 
     def highlight_color(self, record: Record) -> Colour:
-        level = record.extract(self.level_key.name)
-        return self.colours.get(level)
+        return self.colour(record.extract(self.level_key))
 
-    def add_multiline_keys(self: P, keys: typing.Sequence[str]) -> P:
-        multiline_keys = Key.from_strings(keys)
+    def add_multiline_keys(self: P, multiline_keys: typing.Sequence[str]) -> P:
         return self.replace(multiline_keys=(*self.multiline_keys, *multiline_keys))
 
+    def is_multiline_json(self) -> bool:
+        return self.multiline_json
+
+    def colour(self, value: Value) -> Colour:
+        if value in self.colours:
+            return self.colours[value]
+
+        if isinstance(value, str):
+            normalised = value.casefold()
+            if normalised in self.colours:
+                return self.colours[normalised]
+
+        return Colour()
+
 
-@dataclasses.dataclass(frozen=True)
 class RawPattern(Pattern):
     def format_record(self, record: Record) -> str:
         return self.format_message(record)
@@ -79,7 +96,6 @@ class RawPattern(Pattern):
         return json.dumps(record.data, indent=None)
 
 
-@dataclasses.dataclass(frozen=True)
 class TemplatePattern(Pattern):
     format: str = "{{__message__}}"
 
@@ -87,14 +103,13 @@ class TemplatePattern(Pattern):
         return self.highlight_color(record).style(self.format.format_map(record))
 
 
-@dataclasses.dataclass(frozen=True)
 class KeyValuePattern(Pattern):
     # Priority keys are rendered first, and always rendered.
-    priority_keys: typing.Sequence[Key] = ()
+    priority_keys: typing.Sequence[str] = ()
 
     # Removed keys have been removed from 'priority_keys' and 'multiline_keys',
     # but also need to be removed from the unknown keys in each line.
-    removed_keys: typing.Sequence[Key] = ()
+    removed_keys: typing.Sequence[str] = ()
 
     def format_message(self, record: Record) -> str:
         colour = self.highlight_color(record)
@@ -106,59 +121,54 @@ class KeyValuePattern(Pattern):
             self.level_key,
         }
 
-        unknown_keys: typing.Sequence[Key]  # Retain the record's existing order.
+        unknown_keys: typing.Sequence[str]  # Retain the record's existing order.
         unknown_keys = [k for k in record.ordered_keys() if k not in known_keys]
         format_keys = itertools.chain(self.priority_keys, unknown_keys)
 
         pairs = self._record_pairs(record, format_keys)
-        formatted_pairs = (k.format_pair(v, colour) for k, v in pairs)
+        formatted_pairs = (self._format_pair(k, v, colour) for k, v in pairs)
         return " ".join(formatted_pairs)
 
+    @staticmethod
+    def _format_pair(key: str, value: Value, colour: Colour) -> str:
+        k = f"{key}="
+        v = repr(value)
+
+        if colour:
+            k = Colour(fg="white").style(k)
+            v = colour.style(v)
+
+        return f"{k}{v}"
+
     def _record_pairs(
-        self, record: Record, keys: typing.Iterable[Key]
-    ) -> typing.Iterable[typing.Tuple[Key, RecordValue]]:
+        self, record: Record, keys: typing.Iterable[str]
+    ) -> typing.Iterable[typing.Tuple[str, RecordValue]]:
         """
         Iterate over key=value pairs for specific keys in a record.
 
         A single key might return multiple pairs if it's value is a mapping.
         """
         for key in keys:
-            value = record.extract(key.name)
+            value = record.extract(key)
             if isinstance(value, dict):
-                yield from self._nested_pairs(parent=key.name, data=value)
+                yield from self._nested_pairs(parent=key, data=value)
             elif value is not None:
                 yield key, value
 
     def _nested_pairs(
         self, parent: str, data: typing.Mapping[RecordKey, RecordValue]
-    ) -> typing.Iterable[typing.Tuple[Key, RecordValue]]:
+    ) -> typing.Iterable[typing.Tuple[str, RecordValue]]:
         """Iterate over pairs in a mapping, prefixing each key with a "parent" key."""
         for k, v in data.items():
             nested_key = ".".join((parent, k))
             if isinstance(v, dict):
                 yield from self._nested_pairs(nested_key, v)
             elif v is not None:
-                yield Key(nested_key), v
-
-    @staticmethod
-    def format_key(key: str) -> str:
-        return f"{key}="
-
-    @staticmethod
-    def format_value(value: str) -> str:
-        return repr(value)
-
-    def replace_keys(self, **kwargs: typing.Sequence[str]) -> "KeyValuePattern":
-        changes = {a: Key.from_strings(keys) for a, keys in kwargs.items()}
-        return self.replace(**changes)
-
-    def replace_level_key(self, key: typing.Optional[str]):
-        return self.replace(level_key=Key.from_string(key) if key is not None else None)
+                yield nested_key, v
 
     def remove_keys(self, keys: typing.Sequence[str]) -> "KeyValuePattern":
-        removed_keys = Key.from_strings(keys)
         return self.replace(
-            removed_keys=[*self.removed_keys, *removed_keys],
-            multiline_keys=[k for k in self.multiline_keys if k not in removed_keys],
-            priority_keys=[k for k in self.priority_keys if k not in removed_keys],
+            removed_keys=[*self.removed_keys, *keys],
+            multiline_keys=[k for k in self.multiline_keys if k not in keys],
+            priority_keys=[k for k in self.priority_keys if k not in keys],
         )
